<?php
/*
Plugin Name: WordPress HTTPS
Plugin URI: http://mvied.com/projects/wordpress-https/
Description: This plugin ensures that <em>all</em> elements on a page are accessed through HTTPS if the page is being accessed via HTTPS.
Author: Mike Ems
Version: 0.5.1
Author URI: http://mvied.com/
*/

if ( !class_exists('WordPressHTTPS') ) {
 class WordPressHTTPS {
 	// Deafault Options
 	private $options_default = array(
  	'wordpress-https_externalurls' => 0, // Option to change external URL's to HTTPS
  	'wordpress-https_bypass'       => 0 // Bypass option to check if external elements can be loaded via HTTPS
  );

  function __construct() {
   if ( is_admin() ) {
    add_action('admin_menu', array(&$this, 'menu'));

   	foreach ( $this->options_default as $option => $value ) {
   		if ( get_option($option) === FALSE ) {
   			add_option($option, $value);
   		}
   	}
   }
   add_action('plugins_loaded', array(&$this, 'buffer_start'));
   add_action('wp_footer', array(&$this, 'buffer_end'));
  }

  function menu() {
   add_options_page('WordPress HTTPS Settings', 'WordPress HTTPS', 'manage_options', 'wordpress-https', array(&$this, 'settings'));
  }

  function settings() {
   if ( !current_user_can('manage_options') ) {
    wp_die( __('You do not have sufficient permissions to access this page.') );
   }

   if ( $_SERVER['REQUEST_METHOD'] === 'POST' ) {
    $errors = array();

    if ( $_POST['wordpress-https_bypass'] == 1 ) {
     update_option('wordpress-https_bypass', 1);
    } else {
     update_option('wordpress-https_bypass', 0);
    }

    if ( $_POST['wordpress-https_externalurls'] == 1 ) {
     if ( get_option('wordpress-https_bypass') != 1 && ini_get('allow_url_fopen') != 1 ) {
      $errors[] = 'PHP configuration error: allow_url_fopen must be enabled to allow the conversion of external elements to HTTPS.';
      update_option('wordpress-https_externalurls', 0);
     } else {
      update_option('wordpress-https_externalurls', 1);
     }
    } else {
     update_option('wordpress-https_externalurls', 0);
    }

   }
?>
 <div class="wrap">
  <div id="icon-options-general" class="icon32"><br /></div>
  <h2>WordPress HTTPS Settings</h2>
<?php
   if ( $_SERVER['REQUEST_METHOD'] === 'POST' ) {
    if ( sizeof( $errors ) > 0 ) {
     echo "  <ul id=\"message\">\n";
     foreach ( $errors as $error ) {
      echo "   <li class=\"error\"><p>".$error."</p></li>\n";
     }
     echo "  </ul>\n";
    } else {
     echo "  <div class=\"updated fade\" id=\"message\"><p><strong>Settings saved.</strong></p></div>\n";
    }
   }
?>

  <form name="form" action="options-general.php?page=wordpress-https" method="post">
   <?php settings_fields('wordpress-https'); ?>
   <table class="form-table">
    <tr valign="top">
     <th scope="row">External HTTPS Elements</th>
     <td>
      <fieldset>
       <legend class="screen-reader-text"><span>Attempt to automatically convert external elements to HTTPS.</span></legend>
       <label for="wordpress-https_externalurls"><input name="wordpress-https_externalurls" type="checkbox" id="wordpress-https_externalurls" value="1"<?php echo ((get_option('wordpress-https_externalurls')) ? ' checked="checked"' : ''); ?> /> Attempt to automatically convert external elements to HTTPS.</label>
       <p class="description">Warning: This option checks that the external element can be loaded via HTTPS while the page is loading. Depending on the amount of external elements, this could affect the load times of your pages.</p>
      </fieldset>
     </td>
    </tr>
    <tr valign="top">
     <th scope="row">Bypass External Check</th>
     <td>
      <fieldset>
       <legend class="screen-reader-text"><span>Disable the option to check if an external element can be loaded over HTTPS.</span></legend>
       <label for="wordpress-https_bypass"><input name="wordpress-https_bypass" type="checkbox" id="wordpress-https_bypass" value="1"<?php echo ((get_option('wordpress-https_bypass')) ? ' checked="checked"' : ''); ?> /> Disable the option to check if an external element can be loaded over HTTPS.</label>
       <p class="description">Bypassing the HTTPS check for external elements may cause elements to not load at all. Only enable this option if you know that all external elements can be loaded over HTTPS.</p>
      </fieldset>
     </td>
    </tr>
    <tr>
     <td>
     </td>
    </tr>
   </table>
   <p class="submit"><input type="submit" name="Submit" value="Save Changes" class="button-primary" /></p>
   <p>This plugin was created by <a href="http://mvied.com/" target="_blank">Mike Ems</a>.</p>
   <p>If you found my plugin useful, please <a href="https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&hosted_button_id=6ZL95VTJ388HG" target="_blank">donate</a>.</p>
   <p>If you need technical assistance, want to provide feedback, or just want to say thanks, <a href="http://wordpress.org/tags/wordpress-https#postform" target="_blank">drop me a line</a>.</p>
  </form>
 </div>
<?php
  }

  function process($buffer) {
   if ( !empty($_SERVER['HTTPS']) || $_SERVER['SERVER_PORT'] == 443 ) {
    $httpsUrl = str_replace('http://', 'https://', get_bloginfo('siteurl'));
    $httpUrl  = str_replace('https://', 'http://', get_bloginfo('siteurl'));

    preg_match_all('/\<(script|link|img)(.*)http:\/\/[-\w\.]+[^\"\']*/im',$buffer,$matches);
    foreach ($matches[0] as $match) {
     if ( ( strpos($match, 'link') !== false && strpos($match, 'stylesheet') !== false ) || strpos($match, 'script') !== false || strpos($match, 'img') !== false ) {
      if ( strpos($match,$httpUrl) !== false ) {
       $buffer = str_replace($match, str_replace('http://', 'https://', $match), $buffer);
      } elseif ( get_option('wordpress-https_externalurls') == '1' ) {
       preg_match_all('/http:\/\/[-\w\.]+[^\"\']*/im', $match, $urlMatches);
       foreach ( $urlMatches[0] as $urlMatch ) {
        if ( get_option('wordpress-https_bypass') == 1 ) {
         $buffer = str_replace($urlMatch, str_replace('http://', 'https://', $urlMatch), $buffer);
        } else if (@file_get_contents(str_replace('http://', 'https://', $urlMatch))) {
         $buffer = str_replace($urlMatch, str_replace('http://', 'https://', $urlMatch), $buffer);
        }
       }
      }
     }
    }
   }
   return $buffer;
  }

  function buffer_start() {
   if ( get_option('wordpress-https_externalurls') == 1 && get_option('wordpress-https_bypass') != 1 ) {
    @ini_set('allow_url_fopen', 1);
   }
   ob_start(array(&$this, 'process'));
  }

  function buffer_end() {
   ob_end_flush();
  }

 } // End WordPressHTTPS Class
}

if ( class_exists('WordPressHTTPS') ) {
 $wordpress_https = new WordPressHTTPS();
}
?>